/**************************************************
opengate Mac addr auth program 
 module to get mac address

Copyright (C) 2011 Opengate Project Team
Written by Yoshiaki Watanabe

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

Email: watanaby@is.saga-u.ac.jp
**************************************************/

#include	"opengatemd.h"

int isFoundInArpOrNdpTable(char* ipAddress, char* macAddress){

  char macAddrInTable[ADDRMAXLN];
  
  /* if IPv4 address, get macAddr from arp */
  if(strchr(ipAddress, ':')==NULL){
    GetMacAddrFromArp(ipAddress, macAddrInTable);
  }

  /* if IPv6 address, get macAddr from ndp */
  else{
    GetMacAddrFromNdp(ipAddress, macAddrInTable);
    ReFormatMacAddr(macAddrInTable);
  }

  /* if mac in arp/ndp table == mac in packet, return true */
  if(strncmp(macAddrInTable, macAddress, ADDRMAXLN)==0) return TRUE;
  else return FALSE;
}

/*******************************************************************/
/* get MAC address for ipAddr (nnn.nnn.nnn.nnn) by arp request */
/*******************************************************************/
int getMacAddrFromArp(char *ipAddr4, char* macAddr4)
{
  char buf[BUFFMAXLN];
  char *startp;
  char *endp;
  FILE *fpipe;

  macAddr4[0]='?';
  macAddr4[1]='\0';

  /* exec arp */
  if( (fpipe=Popenl(1, "r", GetConfValue("ArpPath"), "-n", 
		    ipAddr4, (char *)0)) == NULL){ 
    err_msg("ERR at %s#%d: exec arp -n error",__FILE__,__LINE__);
    return -1;
  }
  
  /* get arp response */  
  if(fgets(buf, BUFFMAXLN, fpipe)==NULL){
    err_msg("ERR at %s#%d: readin error",__FILE__,__LINE__);
    Pclose(fpipe);
    return -1;
  } 

  /* close pipe */
  Pclose(fpipe);

  /* arp response takes following format           */
  /* "? (133.49.22.1) at 8:0:20:a5:4:62 [ethernet]"*/
  /* get MAC address from above string             */
  if((startp=strstr(buf," at "))==NULL) return -1;

  startp=startp+4;
  if((endp=strstr(startp, " "))==NULL) return -1;

  /* cut off string at endp */
  *endp='\0';

  /* return if invalid address */
  if((strstr(startp, ":"))==NULL) return -1;

  /* save to macAddr4 */
  strncpy(macAddr4, startp, ADDRMAXLN);
    
  return 0;
}

/**********************************************************************/
/* get MAC address for clientAddr (nnnn:nnnn::nnnn:nnnn) by ndp entry */
/**********************************************************************/
int getMacAddrFromNdp(char *ipAddr6, char* macAddr6)
{
  FILE *fpipe;
  char buf[BUFFMAXLN];
  char *startp;

  macAddr6[0]='?';
  macAddr6[1]='\0';

  /* exec proc */
  if((fpipe=Popenl(1, "r", GetConfValue("NdpPath"),"-na",(char *)0)) == NULL){ 
    err_msg("ERR at %s#%d: exec ndp -na error",__FILE__,__LINE__);
    return -1;
  }

  /* get ndp response */
  /* skip first title line */
  if(fgets(buf, BUFFMAXLN, fpipe)==NULL){
    err_msg("ERR at %s#%d: readin error",__FILE__,__LINE__);
    Pclose(fpipe);
    return -1;
  }
  
  /* arp response takes following format           */
  /* "[IPv6 Addr] [Mac] [InterfaseID] [Expire] [Status] [Flags] [Prbs]" */
  /* get MAC address from above string             */
  while(fgets(buf, BUFFMAXLN, fpipe)!=NULL){
    if(strstr(buf,ipAddr6)==NULL)continue;
    startp = strtok(buf," "); /* first token */
    startp = strtok(NULL," "); /* second token */

    /* skip if invalid address */
    if(strstr(startp, ":")==NULL)continue;

    strcpy(macAddr6,startp);
  }

  Pclose(fpipe);
  
  return 0;
}

/****************************************/
/****************************************/
int GetMacAddrFromArp(char *ipAddr4, char* macAddr4)
{
  int ret;

  if(debug>1) err_msg("DEBUG:=>getMacAddrFromArp(%s,)",ipAddr4);
  ret=getMacAddrFromArp(ipAddr4,macAddr4);
  if(debug>1) err_msg("DEBUG:(%d)<=getMacAddrFromArp(,%s)",ret,macAddr4);

  return ret;
}  

int GetMacAddrFromNdp(char *ipAddr6, char* macAddr6)
{
  int ret;

  if(debug>1) err_msg("DEBUG:=>getMacAddrFromNdp(%s,)",ipAddr6);
  ret=getMacAddrFromNdp(ipAddr6,macAddr6);
  if(debug>1) err_msg("DEBUG:(%d)<=getMacAddrFromNdp(,%s)",ret,macAddr6);

  return ret;
}

int IsFoundInArpOrNdpTable(char* ipAddress, char* macAddress){
  int ret;

  if(debug>1) err_msg("DEBUG:=>isFoundInArpOrNdpTable(%s,%s)",ipAddress,macAddress);
  ret=isFoundInArpOrNdpTable(ipAddress,macAddress);
  if(debug>1) err_msg("DEBUG:(%d)<=getMacAddrFromNdp()",ret);

  return ret;
}
